package org.sample.nio.connection.handler;


import java.nio.channels.SelectionKey;

import org.sample.nio.QuoteManager;

import com.sun.grizzly.SelectorHandler;

/**
 * This class contain the logic to parse the request
 * from the client.
 * 
 * There is one Thread by client connected.
 * 
 * @author Sebastien Dionne
 *
 */
public class ClientConnectionHandler {
	
	protected QuoteManager f_manager;
	protected SelectionKey f_key;
	protected SelectorHandler f_selectorHandler;
	
	protected boolean f_shutdown = false;
	
	/**
	 * manage the connection with the client
	 * @param manager le QuoteManager
	 */
	public ClientConnectionHandler(QuoteManager manager, SelectionKey key, SelectorHandler selectorHandler){
		f_manager = manager;
		f_key = key;
		f_selectorHandler = selectorHandler;
	}
	
	
	public SelectionKey getKey() {
		return f_key;
	}

	public void setKey(SelectionKey f_key) {
		this.f_key = f_key;
	}

	public SelectorHandler getSelectorHandler() {
		return f_selectorHandler;
	}

	public void setSelectorHandler(SelectorHandler handler) {
		f_selectorHandler = handler;
	}


	/**
	 * close the client connection
	 */
	public synchronized void close(){
		
		System.out.println("ClientConnection close");
		if(f_shutdown){
			return;
		}
		 // No more bytes can be read from the channel
		f_selectorHandler.getSelectionKeyHandler().cancel(f_key);
		
		// unsubcribe to quotefeed
		//f_manager.unRegisterClientConnectionHandler(this);
		f_manager.unsubcribeClient(this);
		
		f_shutdown = true;
	}
	
}
