package org.sample.nio.connection.handler;


import java.io.IOException;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;

import org.sample.nio.QuoteManager;
import org.sample.nio.connection.handler.filter.QuoteQueryManagerFilter;
import org.sample.nio.connection.handler.filter.QuoteQueryProtocolFilter;

import com.sun.grizzly.BaseSelectionKeyHandler;
import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultPipeline;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.DefaultSelectionKeyHandler;
import com.sun.grizzly.Pipeline;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolChainInstanceHandler;
import com.sun.grizzly.TCPSelectorHandler;

/**
 * This class listen for incoming connection
 * 
 * @author Sebastien Dionne
 *
 */
public class GrizzlyConnectionListener implements Runnable {
	
	protected String f_name;
	protected int f_port;
	protected QuoteManager f_quoteManager;
	protected boolean f_shutdown;
	protected ThreadGroup f_threadGroup;
	protected ServerSocketChannel f_serverSocket;
	protected ServerSocketChannel f_serverSocketChannel;
	protected Selector f_selector;
	
	protected Controller f_controller;
	
	/**
	 * @return the quoteManager
	 */
	public QuoteManager getQuoteManager() {
		return f_quoteManager;
	}

	/**
	 * @param quoteManager the quoteManager
	 */
	public void setQuoteManager(QuoteManager quoteManager) {
		this.f_quoteManager = quoteManager;
	}
	
	/**
	 * 
	 * @param port port for the incomming connection
	 */
	public void setPort(int port){
		f_port = port;
	}
	
	/**
	 * 
	 * @return the listening port
	 */
	public int getPort(){
		return f_port;
	}

	/**
	 * Init
	 */
	public void init(){
		
		System.out.println("listening for incomming TCP Connections on port : " + f_port);
		try {
			
			f_controller = new Controller();
			TCPSelectorHandler tcpSelectorHandler = new TCPSelectorHandler();
			tcpSelectorHandler.setPort(f_port);

			Pipeline pipeline = new DefaultPipeline();
			pipeline.setMaxThreads(5);
			
			f_controller.setPipeline(pipeline);
			
           tcpSelectorHandler.setSelectionKeyHandler(new BaseSelectionKeyHandler());
           
           f_controller.addSelectorHandler(tcpSelectorHandler);
           
           QuoteQueryProtocolFilter protocolParser = new QuoteQueryProtocolFilter();
           QuoteQueryManagerFilter quoteManagerFilter = new QuoteQueryManagerFilter(f_quoteManager);
           
           final ProtocolChain protocolChain = new DefaultProtocolChain();
           protocolChain.addFilter(protocolParser);
           protocolChain.addFilter(quoteManagerFilter);
           ((DefaultProtocolChain) protocolChain).setContinuousExecution(true);


           ProtocolChainInstanceHandler pciHandler = new DefaultProtocolChainInstanceHandler() {

               public ProtocolChain poll() {

                   return protocolChain;
               }

               public boolean offer(ProtocolChain protocolChain) {
                   return false;

               }
           };

           f_controller.setProtocolChainInstanceHandler(pciHandler);
           try {
        	   f_controller.start();
           } catch (IOException e) {
               e.printStackTrace();
           }
           
			
		} catch (Exception e) {
			System.exit(-10);
		}
	}

	/**
	 * @return the shutdown
	 */
	public boolean isShutdown() {
		return f_shutdown;
	}

	/**
	 * shutdown the socket listener
	 */
	public void shutdown() {
		f_shutdown = true;
		// A TESTER
		f_threadGroup.interrupt();
	}
	
	/**
	 * processing
	 * 
	 * - Open socket
	 * - a new Thread (ClientConnectionHandler)
	 * - start the new thread
	 */
	public void run() {

		init();
		
		
	}
	
	
}
